<?php
/**
 * Future funcionality to be release in Version 2.2.04+
 * @author Payment Plugins
 *
 */
class BT_Manager {
	
	public $_attributes = array();
	public $environment = 'sandbox';
	//public $merchantAccountConfig;
	public static $_instance = null;
	
	public function __construct(){
		self::$_instance = $this;
		$this->init();
	}
	
	private function init(){
		$this->_set('merchantAccountConfig', $this->get_payments_config('braintree_merchant_account_id_config'));
	}
	
	public function _set($key, $value){
		$this->_attributes[$key] = $value;
	}
	
	public function __get($key){
		if(array_key_exists($key, $this->_attributes)){
			return $this->_attributes[$key];
		}
		return '';
	}
	
	public static function add_merchantAccount(){
		if(isset($_POST['add_braintree_merchant_account']) && !empty($_POST['add_braintree_merchant_account'])){
			$config = BT_Manager()->merchantAccountConfig;
			$config['production'][] = array('id'=>'', 'currency'=>'');
			BT_Manager()->update_payments_config('braintree_merchant_account_id_config', $config);
			wp_redirect(get_admin_url().'admin.php?page=braintree-payment-settings&tab=braintree-merchant-account-config');
		}
		if(isset($_POST['add_braintree_sandbox_merchant_account']) && !empty($_POST['add_braintree_sandbox_merchant_account'])){
			$config = BT_Manager()->merchantAccountConfig;
			$config['sandbox'][] = array('id'=>'', 'currency'=>'');
			BT_Manager()->update_payments_config('braintree_merchant_account_id_config', $config);
			wp_redirect(get_admin_url().'admin.php?page=braintree-payment-settings&tab=braintree-merchant-account-config');
		}
	}
	
	public static function deleteMerchantAccount(){
		if(isset($_POST['delete_production_merchant_account']) && !empty($_POST['delete_production_merchant_account'])){
			$index = substr($_POST['delete_production_merchant_account'], -1);
			$config = BT_Manager()->merchantAccountConfig();
			if(isset($config['production'][$index])){
				unset($config['production'][$index]);
				BT_Manager()->update_payments_config('braintree_merchant_account_id_config', $config);
				wp_redirect(get_admin_url().'admin.php?page=braintree-payment-settings&tab=braintree-merchant-account-config');
			}
		}
		if(isset($_POST['delete_sandbox_merchant_account']) && !empty($_POST['delete_sandbox_merchant_account'])){
			$index = substr($_POST['delete_sandbox_merchant_account'], -1);
			$config = BT_Manager()->merchantAccountConfig();
			if(isset($config['sandbox'][$index])){
				unset($config['sandbox'][$index]);
				BT_Manager()->update_payments_config('braintree_merchant_account_id_config', $config);
				wp_redirect(get_admin_url().'admin.php?page=braintree-payment-settings&tab=braintree-merchant-account-config');
			}
		}
	}
	
	public function get_payments_config($option){
		$option = base64_encode($option);
		return maybe_unserialize(base64_decode(get_option($option)));
	}
	
	public function update_payments_config($option, $value){
		$option = base64_encode($option);
		update_option($option, base64_encode(maybe_serialize($value)));
	}
	
	public function delete_payments_config($option, $value){
		$option = base64_encode($option);
		delete_option($option, base64_encode(maybe_serialize($value)));
	}
	/**
	 * Returns the Merchant Account ID based on the currency.
	 * @param string $currency
	 */
	public function merchantAccountConfig($environment = null){
		if($environment == null){
			return $this->merchantAccountConfig;
		}
		if(isset($this->merchantAccountConfig[$environment])){
			return $this->merchantAccountConfig[$environment];
		}
		return array();
	}
	
	/**
	 * Gets the default merchant account
	 * @param array
	 */
	public function getDefaultMerchantAccount($environment = 'sandbox'){
		if(isset($this->merchantAccountConfig['default'][$environment])){
			return $this->merchantAccountConfig['default'][$environment];
		}
		return array('USD');
	}
	
	/**
	 * Update the database so it contains the proper merchant account Id
	 */
	public static function versionUpdater(){
		$config = BT_Manager()->get_payments_config('braintree_merchant_account_id_config');
		if(isset($config['production']['merchant_account_id'])){
			$currency = get_woocommerce_currency();
			$newConfig = array(
					'production'=>array(
							array('id'=>$config['production']['merchant_account_id'], 'currency'=>$currency)
					),
					'sandbox'=>array(
							array('id'=>isset($config['sandbox']['merchant_account_id']) ? $config['sandbox']['merchant_account_id'] : '', 'currency'=>$currency)
							
					)
				);
			BT_Manager()->update_payments_config('braintree_merchant_account_id_config', $newConfig);
		}
	}
	
	public function get_currenciesHTML($currency){
		foreach(Braintree_Currencies::get_currencies() as $current_currency => $symbol){
			?>
			<option value="<?php echo $current_currency?>" <?php if($currency === $current_currency){?>selected="selected"<?php }?>><?php echo $current_currency.' - '.$symbol?></option>
			<?php 
					
		}
	}
	
	public static function checkPHPVersion(){
		if(PHP_VERSION < 5.4){
			add_action('admin_notices', 'BT_Manager::displayPHPVersionError');
		}
	}
	
	public static function displayPHPVersionError(){
		$message = sprintf('Your PHP Version is %s but Braintree requires PHP Version 5.4 or greater. Please update
				your PHP Version to activate this plugin.', PHP_VERSION);
		echo '<div class="error"><p>'.$message.'</p></div>';
	}
}
function BT_Manager(){
	if(BT_Manager::$_instance){
		return BT_Manager::$_instance;
	}
	return new BT_Manager();
}
add_action('admin_init', 'BT_Manager::add_merchantAccount');
add_action('admin_init', 'BT_Manager::deleteMerchantAccount');
register_activation_hook(__FILE__, 'BT_Manager::checkPHPVersion');
?>