<?php

define('WC_BRAINTREE_GATEWAY', plugin_dir_path(__FILE__));
define('WC_BRAINTREE_SCRIPT_PATH', plugin_dir_url(__FILE__).'assets/');

function init_wc_braintree_payment_gateway(){
		if(class_exists('WC_Payment_Gateway')) {
			class Braintree_For_Woocommerce extends WC_Payment_Gateway {
				
				public $environment;
				
				private $merchantId;
				
				private $private_key;
				
				private $public_key;
				
				public $form_type;
				
				public $payment_form_name;
				
				public $config;
				
				public $paypal_enabled;
				
				public $method_title;
				
				public $order_status;
				
				public $subscription_id;
				
				public $merchantAccountId;
				
				public $only_paypal;
				
				public $order_prefix;
				
				public $accepted_paymentMethods;
				
				public $log;
				
				public static $is_woocommerce_subscription_active;
				
				public function __construct(){
					$this->id = 'braintree_payment_gateway';
					
					$this->icon = '';
					//$this->title = __('Credit Card', 'Braintree_Woocommerce');
					$this->method_title = __('Braintree For Woocommerce', 'woocommerce');
					$this->has_fields = true;
					
					$this->supports = array('subscriptions',
					'products',
					'subscription_cancellation',
					//'subscription_reactivation',
					'multiple_subscriptions',
					'subscription_amount_changes',
					'subscription_payment_method_change',
					'subscription_date_changes',
					'default_credit_card_form',
					'refunds',
					'pre-orders',
					'subscription_payment_method_change_admin',
					'gateway_scheduled_payments',
					'subscription_payment_method_change_customer'		
					);
					$this->log = new WC_Logger();
					
					$this->init_form_fields();
				
					$this->init_settings();
					
					$this->load_options();
					
					$this->run_configuration();
					
					$this->load_braintree_scripts();
					
					$this->log = new BraintreeTransactionLog();
					
					add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
					
				}
				
				private function load_options(){
					
		
					if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')
						$_SERVER['HTTPS'] = 'on';
					
					$license_status = get_braintree_payments_config('braintree_payments_license_status');
					$this->config['api_keys'] = maybe_unserialize(get_braintree_payments_config('braintree_api_keys_config'));
					$this->config['merchant_accounts'] = maybe_unserialize(get_braintree_payments_config('braintree_merchant_account_id_config'));
					$this->config['woopayments'] = maybe_unserialize(get_braintree_payments_config('braintree_payments_woocommerce_config'));
					$this->config['subscription_config'] = get_braintree_payments_config('braintree_subscription_config');
					if(isset($this->config['api_keys']['environment'])){
						$production = ($this->config['api_keys']['environment'] === 'production') ? true : false;
					}
					if($license_status == 'inactive'){
						check_braintree_donations_license();
						$license_status = get_braintree_payments_config('braintree_payments_license_status');
						$production = ($license_status == 'inactive') ? false : true;
					}
					
				 	if($production && !is_ssl()){
						$production = false;
					}
					
					$this->environment = ($production) ? 'production' : 'sandbox';
					if(isset($this->config['api_keys'])){
						$this->merchantId = isset($this->config['api_keys'][$this->environment]['merchantId']) ? $this->config['api_keys'][$this->environment]['merchantId'] : '';
						$this->public_key = isset($this->config['api_keys'][$this->environment]['public_key']) ? $this->config['api_keys'][$this->environment]['public_key'] : '';
						$this->private_key  = isset($this->config['api_keys'][$this->environment]['private_key']) ? $this->config['api_keys'][$this->environment]['private_key'] : '';
					}
					$this->merchantAccountId = isset($this->config['merchant_accounts'][$this->environment]['merchant_account_id']) ?
								 $this->config['merchant_accounts'][$this->environment]['merchant_account_id'] : '';
					if(isset($this->config['woopayments'])){
						$this->paypal_enabled = $this->get_setting($this->config['woopayments'], 'woopayments_paypal_enabled') == 'true' ? true : false;
						$this->only_paypal = $this->get_setting($this->config['woopayments'], 'only_paypal') == 'true' ? true : false;
						$this->form_type = $this->get_setting($this->config['woopayments'], 'payment_form_type');
						$this->payment_form_name = $this->get_setting($this->config['woopayments'], 'payment_form');
						$this->order_status = (! empty($this->config['woopayments']['order_status'])) ? 
							$this->config['woopayments']['order_status'] : 'wc-completed';
						$this->order_prefix = $this->get_setting($this->config['woopayments'], 'order_prefix');
						$this->accepted_paymentMethods = $this->get_setting($this->config['woopayments'], 'accepted_paymentmethods');
						$this->title = $this->get_setting($this->config['woopayments'], 'payment_title');
					}
				}
				
				private function run_configuration(){
					if(class_exists('Braintree_Configuration')){
						Braintree_Configuration::environment($this->environment);
						Braintree_Configuration::merchantId($this->merchantId);
						Braintree_Configuration::publicKey($this->public_key);
						Braintree_Configuration::privateKey($this->private_key);
					}
				}
				
				private function get_setting($array, $value){
					if(isset($array[$value])){
						return $array[$value];
					}
					else {
						$array[$value] = '';
					}
					return $array[$value];
				}
				
				public function admin_options(){
					?>
					<div>
						<ul>
							<li><a href="<?php echo get_admin_url().'admin.php?page=braintree-payment-settings'?>">Braintree Settings</a></li>
							<li><a href="<?php echo get_admin_url().'admin.php?page=braintree-payments-woocommerce-settings'?>">Woocommerce Payments</a></li>
							<li><a href="<?php echo get_admin_url().'admin.php?page=braintree-payments-instructions'?>">Instructions</a></li>
							<li><a href="<?php echo get_admin_url().'admin.php?page=braintree-payments-activation'?>">Activate License</a></li>
							<li><a href="<?php echo get_admin_url().'admin.php?page=braintree-subscription-config'?>">Subscriptions</a></li>
							<li><a href="<?php echo get_admin_url().'admin.php?page=braintree-transaction-log'?>">Transaction Log</a></li>
							
						</ul>
				
					</div>
					<?php 
				}
				
				public function get_client_token($user_customer = true){
					if($user_customer && $user = wp_get_current_user()){
						return Braintree_ClientToken::generate(array('customerId'=>$user->ID));
					}
					try{
						return Braintree_ClientToken::generate();
					}
					catch(Exception $e){
						return false;
					}
				}
				
				public function process_payment($order_id){
					return apply_filters('process_customer_payment', $order_id, $this);
				}
				
				public function can_refund_order(WC_Order $order){
					return $order && $order->get_transaction_id();
				}
				
				public function process_refund( $order_id, $amount = null, $reason = '' ){
					$order = new WC_Order($order_id);
					if(! $this->can_refund_order($order)){
						$this->log->writeToLogFile(sprintf('Order %s cannot be refunded via the admin console because there is no transaction id on record', $order->id));
						/* if($this->has_subscriptions($order)){
							return new WP_Error('error', sprintf('Order %s has subscription orders associated with it. You must refund a specific subscription order.', $order->id));
						} */
						return new WP_Error('error', sprintf('Cannot refund order %s. There is no transaction id associated with the order.', $order->id));
					}
					//refund the order since there is a transaction id.
						
							
								try {
									$result = Braintree_Transaction::refund($order->get_transaction_id(), $amount);
									if($result->success){
										$this->log->writeToLogFile(sprintf('Order %s refund successful. Amount: %s', $order->id, $amount));
									}
									else {
										$this->log->writeToLogFile(sprintf('Order %s could not be refunded. Message: %s', $order->id, $result->message));
										return new WP_Error('error', sprintf('Order %s could not be refunded. Message: %s', $order->id, $result->message));
									}
								} 	
								catch(Exception $e){
									$this->log->writeToLogFile(sprintf('An exception was thrown while refunding order %s. Message: %s', $order->id, $e->getMessage()));
								return WP_Error(sprintf('An exception was thrown while refunding order %s. Message: %s', $order->id, $e->getMessage()));
								} 
					return true;
				}
				
				public function get_paypal_container(){
					return '<div class="paypal-container">
							<div class="paypal-pay"><span>Pay With Paypal</span></div>
							<div id="paypal-container"></div>
							</div>
						<input type="hidden" id="paypal_active"/>
					<script type="text/javascript">
					jQuery(document).ready(function(){
							var paypalCheckout = null;
		  					var $ = jQuery;
		  					var amount = $("#order_amount").val();
		  					var clientToken = "'.send_braintree_client_token(false).'";			
							function initializePaypalCheckout(){
			  					braintree.setup(clientToken, "paypal", {
			  					  	container: "paypal-container",
			  					  	onPaymentMethodReceived: function (response) {
			  					  		$("input[name=\'payment_method_nonce\']").val(response.nonce);
			  					 	},
			  					  onReady: function(integration){
									paypalCheckout = integration;
			  					  }
			  					});
							}
							$(document.body).on("updated_checkout", function(){
								/* if(typeof paypalCheckout === "object" && paypalCheckout !== null){
									paypalCheckout.teardown(function(){
										paypalCheckout = null;
									});
									initializePaypalCheckout();
								} */
							});
						    
							initializePaypalCheckout();
		  				})
					</script>';
				}
				
			public function get_current_braintree_user($user_id){
			try{
			$user = Braintree_Customer::find($user_id);
				return $user;
			}
			catch(Exception $e){
				return false;
			}
		}
		
		public function get_user_cards($braintree_user){
			if($braintree_user){
			if($braintree_user->paymentMethods){
				$count = 0;
			$html =	'<div class="payment-info-container"><div class="use-saved-card"><div><span>Use Saved</span></div><input type="checkbox" id="use_saved_card" name="use_saved_card" value="true"/></div><div class="saved-payment-methods"><select name="existing_card">';
					foreach($braintree_user->creditCards as $card => $creditCard) {
						$html .= '<option name="saved_card" value="'.$count.'">'.$creditCard->cardType.' - '.$creditCard->maskedNumber.'</option>';
					$count++;
					}
				$html .= '</select></div></div>';
			return $html;
			}
		}
		}
				/**
				 * Verify that force SSL option is set to true. If it is not, then do not allow the 
				 * customer to use the payment gateway using production keys. First, the code checks to see if woocommerce
				 * force SSL in checkout is set. If not, the header for 'HTTPS' is checked. If that is also false, then the wordpress
				 * function is_ssl() is called. If the function returns false, then Braintree is loaded using the sandbox public and private
				 * keys
				 */
				private function is_ssl_enabled(){
					$ssl_option = 'yes' == get_option('woocommerce_force_ssl_checkout', true) ? true : false;
					if(!$ssl_option){
							if(!is_ssl()){
							return false;
						}
					}
					return true;
				}
				public function payment_fields(){
					apply_filters('get_braintree_checkout_ui', $this);
		       }
		       public function load_braintree_scripts(){
		       	//wp_enqueue_script('braintree-custom-ui-script', WC_BRAINTREE_SCRIPT_PATH.'js/braintree.js', array('jquery'), '', true );
		       	wp_enqueue_script('braintree-custom-ui-script', 'https://js.braintreegateway.com/v2/braintree.js', array('jquery'), '', true );
		       	wp_enqueue_style('payment-form-style', WC_BRAINTREE_SCRIPT_PATH.'css/'.$this->payment_form_name.'.css');
		       	wp_enqueue_style('braintree-checkout-style', WC_BRAINTREE_SCRIPT_PATH.'css/braintree-checkout.css');
		       }
		  
		  public function get_active_environment(){
		  	if($this->environment == 'sandbox'){
		  		return '<div class="active-environment-message"><span>Sandbox Active</span></div>';
		  	}
		  }
		  
		  public function has_subscriptions(WC_Order $order){
		  	if($subscriptions = wcs_get_subscriptions_for_order($order->id)){
		  		return true;
		  	}
		  }
		  public function get_paypalOnlyCheckout(){
		  	echo '<input type="hidden" id="client_token" value="'.$this->get_client_token(false).'"/>
		  			<input type="hidden" name="payment_type" value=""/>';
		  	echo $this->get_active_environment();
		  	echo '<div class="paypal-container">
							<div class="paypal-pay"><span>Pay With Paypal</span></div>
							<div id="paypal-container"></div>
						</div>
						<input type="hidden" id="paypal_active"/>
					<script type="text/javascript">
		 		jQuery(document).ready(function(){
				 var paypalCheckout = null;
				window.paypalInitialized = false;
				 var $ = jQuery;
					var clientToken = $("#client_token").val();
					form = $("form[name=\'checkout\']").attr("id", "checkout");
				if($("#order_review").length && $("#order_review").is("form")){
		 				if(clientToken){
						braintree.setup(clientToken, "paypal", {container: "paypal-container", 
							onPaymentMethodReceived: function (response){
									$("input[name=\'payment_method_nonce\']").val(response.nonce);
		 							$("#payment_type").val(response.type);
								}, 
							onReady: function(integration){
									paypalCheckout = integration;
		 						}
							});
					
					}
		 		}
		 		else {
		 		function initializePaypalCheckout(){
					if(paypalInitialized === true) {
						return;
					}
					else if(typeof paypalCheckout === "object" && paypalCheckout !== null){
							paypalCheckout.teardown(function(){
		  						paypalCheckout = null;
		  					});
					}
		 			braintree.setup(clientToken, "paypal", {container: "paypal-container", 
							onPaymentMethodReceived: function (response){
		 						$("input[name=\'payment_method_nonce\']").val(response.nonce);
								},
							onReady: function(integration){
		 						paypalCheckout = integration;
		 					}
						});
					paypalInitialized = true;
		 		}
						
					initializePaypalCheckout();
						
		 				$(\'input[name="payment_method"]\').on("change", function(e){
		 					if($("#payment_method_braintree_payment_gateway").is(":checked")){
		 						if(paypalCheckout == null){
									initializePaypalCheckout();
								}
		 					}
		 					else {
								if(paypalCheckout != null){
									paypalCheckout.teardown(function(){
										paypalCheckout = null;
		 							});
									paypalInitialized = false;
		 					}
		 				}});
					
		 		}
		
		  })   
		 		</script>';
		  }
		  
		  public function get_avs_inputFields(){
		  	if(Braintree_For_Woocommerce::$is_woocommerce_subscription_active && WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment) {
				
		  	$html = '<div class="payment-info-container">
								<div class="payment-info"><span>'.__('Billing Address', 'braintree_woocommerce').'</span></div>
								<div class="payment-input"><input type="text" name="billing_address1" value""/></div>
							</div>
							<div class="payment-info-container">
								<div class="payment-info"><span>'.__('Postal Code', 'braintree_woocommerce').'</span></div>
								<div class="payment-input"><input type="text" name="billing_postalcode" value=""/></div>
							</div>
						<div class="payment-info-container">
							<div class="payment-info"><span>'.__('Billing Country', 'braintree_woocommerce').'</span></div>
								<div class="payment-input">
									<select name="billing_country" value="">
									'.$this->get_billingCountries().'
									</select>
								</div>
						</div>';
		  	return $html;
		  	}
		  }
		  
		  public function get_billingCountries(){
		  	$html = '';
		  	foreach(Braintree_Donation_Countries::$countries as $country => $text){
		  		$html .= '<option value="'.$country.'">'.$text.'</option>';
		  	}
		  	return $html;
		  }
		  
		  public function get_accepted_paymentMethods(){
		  	if(! is_array($this->accepted_paymentMethods)){
		  		return '';	
		  	}
		  	$return  = false;
		  	$html = '<div class="accepted-paymentmethods">';
		  	foreach($this->accepted_paymentMethods as $paymentMethod => $value){
		  		if($value === 'yes'){
		  			$return = true;
		  			$src = BRAINTREE_PAYMENTS_ASSETS.'images/'.$paymentMethod.'.png';
		  			$html .= sprintf('<div class="payment-method-image"><img src="%s"></div>', $src);
		  		}
		  	}
		  	$html .= '</div>';
		  	if($return){
		  		return $html;
		  	}
		  }
		  
		  public static function is_currencySwitcherActive(){
		  	return (class_exists('WOOCS_STORAGE'));
		
		  }
		  
		  /**
		   * Fetches the Merchant Account ID configured based on currency.
		   */
		  public function getMerchantAccountId(){
		  	$accounts = BT_Manager()->merchantAccountConfig($this->environment);
		  	if(self::is_currencySwitcherActive()){
		  		if(isset($_SESSION['woocs_current_currency'])){
		  			$currency = $_SESSION['woocs_current_currency'];
		  		}
		  		elseif(isset($_COOKIE['woocs_current_currency'])){
		  			$currency = $_COOKIE['woocs_current_currency'];
		  		}
		  		if(is_array($accounts)){
		  			foreach($accounts as $value){
		  				if($value['currency'] === $currency){
		  					return $value['id'];
		  				}
		  			}
		  		}
		  	}
		  	else {
		  		if(is_array($accounts)){
		  			foreach($accounts as $value){
		  				if($value['currency'] === get_woocommerce_currency())
		  					return $value['id'];
		  			}
		  		}
		  	}
		  	return false;
		  }
		}
	}
}
add_action('plugins_loaded', 'init_wc_braintree_payment_gateway');


function add_braintree_payment_gateway($methods){
	$methods[] = 'Braintree_For_Woocommerce';
	return $methods;
	
}
add_filter('woocommerce_payment_gateways', 'add_braintree_payment_gateway');

function braintree_load_dependencies(){
	if(class_exists('WC_Payment_Gateway')) {
	include_once(WC_BRAINTREE_GATEWAY.'includes/payment-functions.php');
	include_once(WC_BRAINTREE_GATEWAY.'includes/payment-forms.php');
	if(Braintree_For_Woocommerce::$is_woocommerce_subscription_active){
		include_once(WC_BRAINTREE_GATEWAY.'includes/Braintree-WooSubscription.php');
		include_once(WC_BRAINTREE_GATEWAY.'includes/subscription-functions.php');
	}
}
}

function braintree_is_woocommerce_subscription_active(){
	if(class_exists('WC_Payment_Gateway')){
	if(class_exists('WC_Subscriptions')){
		Braintree_For_Woocommerce::$is_woocommerce_subscription_active = true;
		}
	else{
		Braintree_For_Woocommerce::$is_woocommerce_subscription_active = false;
		}
}
}
add_action('plugins_loaded', 'braintree_is_woocommerce_subscription_active', 10);

add_action('plugins_loaded', 'braintree_load_dependencies', 20);

function load_braintree_admin_scripts(){
	wp_enqueue_script('braintree-admin-script', plugin_dir_url(__FILE__).'admin/js/braintree-admin.js');
}
add_action('admin_init', 'load_braintree_admin_scripts');
?>