<?php

function initialize_braintree_subscription_functions(){
	Braintree_WooSubscription::init();
}
add_action('init', 'initialize_braintree_subscription_functions');

class Braintree_WooSubscription {
	
	public static $config;
	public static $environment;
	private static $public_key;
	private static $private_key;
	private static $merchantId;
	public static $merchantAccountId;
	public static $log;
	private static $init_called;
	private static $id = 'braintree_payment_gateway';
	public static $braintree_recurring;
	public static $order_status;
	public static $order_prefix;
	
	public function __construct(){

	}
	
	public static function init(){
		if(self::$init_called){
			return;
		}
		$license_status = get_braintree_payments_config('braintree_payments_license_status');
		self::$config['api_keys'] = maybe_unserialize(get_braintree_payments_config('braintree_api_keys_config'));
		self::$config['merchant_accounts'] = maybe_unserialize(get_braintree_payments_config('braintree_merchant_account_id_config'));
		self::$config['woopayments'] = maybe_unserialize(get_braintree_payments_config('braintree_payments_woocommerce_config'));
		self::$config['subscription_config'] = get_braintree_payments_config('braintree_subscription_config');
			
		$production = isset(self::$config['api_keys']['environment']) && self::$config['api_keys']['environment']=== 'production' ? true : false;
			
		if($license_status == 'inactive' && $production){
			check_braintree_donations_license();
			$license_status = get_braintree_payments_config('braintree_payments_license_status');
			$production = ($license_status == 'inactive') ? false : true;
		}
			
		self::$environment = ($production) ? 'production' : 'sandbox';
		self::$merchantId = isset(self::$config['api_keys'][self::$environment]['merchantId']) ? self::$config['api_keys'][self::$environment]['merchantId'] : '';
		self::$public_key = isset(self::$config['api_keys'][self::$environment]['public_key']) ? self::$config['api_keys'][self::$environment]['public_key'] : '';
		self::$private_key  = isset(self::$config['api_keys'][self::$environment]['private_key']) ? self::$config['api_keys'][self::$environment]['private_key'] : '';
	
		self::$merchantAccountId = isset(self::$config['merchant_accounts'][self::$environment]['merchant_account_id']) ? self::$config['merchant_accounts'][self::$environment]['merchant_account_id'] : '';
	
		self::$braintree_recurring = isset(self::$config['subscription_config']['recurring_billing_type']) && self::$config['subscription_config']['recurring_billing_type'] == 'braintree' ? true : false;
	
	
		self::$order_status = isset(self::$config['woopayments']['order_status']) ? self::$config['woopayments']['order_status'] : 'wc-completed';
		self::$order_prefix = isset(self::$config['subscription_config']['order_prefix']) ? self::$config['subscription_config']['order_prefix'] : '';
	
		
		
		Braintree_Configuration::environment(self::$environment);
		Braintree_Configuration::merchantId(self::$merchantId);
		Braintree_Configuration::publicKey(self::$public_key);
		Braintree_Configuration::privateKey(self::$private_key);
		
		self::$log = new BraintreeTransactionLog();
			
		//Version 1.5 adds the static method to the cancel subscription hook. 
		//add_action('cancelled_subscription_'.self::$id, __CLASS__.'::cancel_braintree_subscription', 10, 2);
		
		
		//Version 2.0+ woocommerce_subscription_pending-cancel_
		//Version 2.0+ adds the static method to the subsciption cancellation hook.
		add_action('woocommerce_subscription_cancelled_'.self::$id, __CLASS__.'::cancel_braintree_subscription', 10, 1);
		
		//Adds the static method to the update order meta hook which is called from the admin order page. 
		if(Braintree_For_Woocommerce::$is_woocommerce_subscription_active){
			add_action('woocommerce_saved_order_items', __CLASS__.'::update_subscription_recurring_amount', 10, 2);
		}
		//Version 2.0+ Adds the static method to the subscription update payment method hook. 
		add_action('woocommerce_subscription_payment_method_updated_to_'.self::$id, __CLASS__.'::change_braintree_subscription_paymentMethod', 10, 2);
		
		//Version 1.5 Registers support for recurring payment payments through the woocommerce subscription API.
		//add_action('scheduled_subscription_payment_'.self::$id, __CLASS__.'::process_recurring_subscriptionPayment', 10, 3);
		
		//Version 2.0+ Registers support for recurring payment payments through the woocommerce subscription API.
		add_action('woocommerce_scheduled_subscription_payment_'.self::$id, __CLASS__.'::process_recurring_subscriptionPayment', 10, 2);
		
		//Version 2.0+
		add_action('woocommerce_my_subscriptions_payment_method', __CLASS__.'::my_subscriptions_payment_method', 10, 2);
		
		//Version 2.0+
		add_action('woocommerce_subscription_failing_payment_method_updated_'.self::$id, __CLASS__.'::failing_payment_method_updated', 10, 1);
		
		//Version 2.0+ renewal order for subscription
		add_action('delete_resubscribe_meta', __CLASS__.'::create_subscription_renewal_order', 10, 2);
		
		//Version 2.0+ Add payment meta fields to Admin order edit page.
		add_filter('woocommerce_subscription_payment_meta', __CLASS__.'::admin_change_subscription_paymentMethod', 10, 2);
		
		//Version 2.0+ validate payment meta entered by admin's.
		add_action('woocommerce_subscription_validate_payment_meta', __CLASS__.'::validate_subscription_payment_meta', 10, 2);
		
		//Version 2.0+ Take the payment meta and update the Woocommerce subscription if one exists.
		add_action('wcs_save_other_payment_meta', __CLASS__.'::save_admin_payment_meta', 10, 4);
		self::$init_called = true;
	}
	
	public static function process_new_subscription($order, Braintree_For_Woocommerce $gateway){
		//check if this is a payment method change. If so, exit method without executing code. 
		if(WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment){
			//If code made it this far, then payment method was changed successfully. 
			return array('result'=>'success', 
					'redirect'=>self::get_paymentMethod_change_redirectUrl()
			);
		}
		if(self::$braintree_recurring){
			return self::initialize_braintree_recurringPayment($order, $gateway);	
		}
		else {
			return self::initialize_subscription_recurringPayment($order, $gateway);
		}
	}
	
	/**
	 * Method that creates the subscription payment method to be used by Woocommerce Subscriptions. This
	 * method does not create a Braintree subscription but rather creates a payment method and ties it to the 
	 * subscriptions so automatic billing can occur.
	 * 
	 * @param WC_Order order object for the shopping cart.
	 */
	private static function initialize_subscription_recurringPayment(WC_Order $order, $gateway){
		//process the subscription using a one time payment;
		$log = new BraintreeTransactionLog();
		
		$use_token = (isset($_POST['use_saved_card'])) ? true : false;
		$token = ($use_token) ? $_POST['existing_card'] : null;
		$nonce = (isset($_POST['payment_method_nonce'])) ? $_POST['payment_method_nonce'] : false;
		
		$args = array('merchantAccountId'=>self::$merchantAccountId);
		
		$user_id = wp_get_current_user()->ID;
		$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
		$payment_token = null;
		
		if(! $braintree_id){
			try{
				$result = Braintree_Customer::create(['firstName'=>$_POST['billing_first_name'],
						'lastName'=>$_POST['billing_last_name']]);
		
				if($result->success){
					$braintree_id = $result->customer->id;
					update_user_meta($user_id, 'braintree_vault_id', $braintree_id);
					$log->writeToLogFile(sprintf('New customer created. User Id: %s, Braintree Id: %s', $user_id, $braintree_id));
				}
				else throw new Exception();
			}
			catch(Exception $e){
				$log->writeToLogFile(sprintf('Customer creation failed during subscription creation. Order ID: %s,  User ID: %s', $order->id, $user_id));
				wc_add_notice(__('Your subscription could not be processed at this time.
							Please try again', 'Braintree_Woocommerce'), 'error');
				return ['result'=>'failure', 'redirect'=>''];
			}
		}
		//process the order total and then save the payment data to each subscription
		$total_amount = $order->get_total();
		
		//The total initial payment is $0, which means we only need a payment token to save against the subscriptions. 
		$subscriptions = wcs_get_subscriptions_for_order($order->id);
		
		//$total_amount = $subscription->get_total();
			
			//There is no initial fee for order so save the payment data to the subscriptions
			if(! $total_amount){
			
				if($token != null){
					//Customer wants to use a previously saved payment method.
					try {
						$customer = Braintree_Customer::find($braintree_id);
						$payment_method = $customer->paymentMethods[$token];
						foreach ($subscriptions as $subscription) {
							self::save_recurring_paymentMethod_meta($payment_method, $subscription);
							self::remove_subscription_from_cart($subscription);
						}
					}
					catch(Braintree_Exception_NotFound $e){
						wc_add_notice(__('The payment method you have selected does not exists in 
							our records. Please select another payment method or re-enter
							your payment method details.', 'Braintree_Woocommerce'), 'error');
						$log->writeToLogFile(sprintf('Customer attempt to use saved payment method faild during 
							subscription creation. User Id: %s, Subscription: %s', $user_id, $subscription->id));
						return ['result'=>'failure', 'redirect'=>''];
					}
				}
				elseif($token == null && $payment_token == null){
					//check if nonce is for a saved payment method.
					try {
						$args = array('paymentMethodNonce'=>$_POST['payment_method_nonce'],
									  'customerId'=>$braintree_id,
									  'options'=>array('failOnDuplicatePaymentMethod'=>true
									  ),
									  'billingAddress'=>array('countryCodeAlpha2'=>$_POST['billing_country'],
									  		'streetAddress'=>$_POST['billing_address_1'],
									  		'postalCode'=>$_POST['billing_postcode']
									  )
						);
						
						$result = Braintree_PaymentMethod::create($args);
						if($result->success){
							foreach($subscriptions as $subscription) {
								self::save_recurring_paymentMethod_meta($result->paymentMethod, $subscription);
								update_post_meta($subscription->id, '_subscription_type', 'woocommerce');
							}
							$payment_method = $result->paymentMethod;
							$payment_token = $payment_method->token;
						}
						else {
							foreach($result->errors->deepAll() as $error){
								if($error->code == '81724'){
									wc_add_notice(__($result->message));
									$log->writeToLogFile(sprintf('User tried to add an existing payment method to the vault. Duplicate payment methods are prohibited. User ID: %s', $user_id));
									return ['result'=>'failure', 'redirect'=>''];
								}
							}
						}
					}
					catch(Exception $e){
						wc_add_notice(__('The payment method you have selected does not exist in
							our records. Please select another payment method or re-enter
							your payment method details.', 'Braintree_Woocommerce'), 'error');
						$log->writeToLogFile($e);
						return ['result'=>'failure', 'redirect'=>''];
				}
			}
		}
		else {
			//logic for when a subscription is to be charged immediately.
			//Use a saved payment method
			if($token != null){
				try {
					$customer = Braintree_Customer::find($braintree_id);
					$payment_token = $customer->paymentMethods[$token]->token;
				}
				catch(Braintree_Exception_NotFound $e){
					wc_add_notice(__('Your payment could not be processed at this time. Please try again.',
							 'Braintree_Woocommerce'), 'error');
					$log->writeToLogFile(sprintf('Customer could not use saved payment method during checkout for subscription %s. User Id: %s', $subscription->id, $user_id));
					return ['result'=>'failure', 'redirect'=>''];
				}
			}
			//Use a new payment method.
		 	else if ($nonce && $token == null && $payment_token == null){
		 			$use_nonce = false;
					try {
						$args = array('paymentMethodNonce'=>$_POST['payment_method_nonce'],
									  'customerId'=>$braintree_id,
									  'options'=>array('failOnDuplicatePaymentMethod'=>true),
									  'billingAddress'=>array('countryCodeAlpha2'=>$_POST['billing_country'],
										'streetAddress'=>$_POST['billing_address_1'],
										'postalCode'=>$_POST['billing_postcode']
								)
						);
					$result = Braintree_PaymentMethod::create($args);
					if($result->success){
						$payment_token = $result->paymentMethod->token;
					}
					else if (! $result->success){
						foreach($result->errors->deepAll() as $error){
							if($error->code == '81724'){
								$use_nonce = true; //Duplicate payment method from dropin ui. Use nonce instead of payment token.
							}
						}
					}
				}
				catch(Exception $e){
					wc_add_notice(__('There was an error validating your payment method. Please verify your entries.',
							 'Braintree_Woocommerce'), 'error');
					return ['result'=>'failure', 'redirect'=>''];
				}
			} 
			try {
				$attrs['amount'] = $total_amount;
				$attrs['orderId'] = self::$order_prefix.$order->id;
				 if(! $use_nonce){
					$attrs['paymentMethodToken'] = $payment_token;
				} 
				 else $attrs['paymentMethodNonce'] = $nonce;
				
				$attrs['customerId'] = $braintree_id;
				$attrs['options'] = ['submitForSettlement'=>true];
				$attrs['billing'] = array('postalCode'=>$_POST['billing_postcode'], 
						'streetAddress'=>$_POST['billing_address_1'], 
						'countryCodeAlpha2'=>$_POST['billing_country']
				);
				
				$result = Braintree_Transaction::sale($attrs);
				
				if($result->success){
					foreach($subscriptions as $subscription){
						self::save_transaction_meta_to_subscription($result, $subscription);
						self::remove_subscription_from_cart($subscription);
						update_post_meta($subscription->id, '_subscription_type', 'woocommerce');
					}
					$log->writeTransactionToLog($result->transaction, 'Subscription');
				}
				else if (! $result->success){
					if($result->errors){
						foreach($result->errors->deepAll() as $error){
							wc_add_notice($error->message, 'error');
						}
					}
					else {
						wc_add_notice($result->message, 'error');
					}
					$log->writeToLogFile(sprintf('Subscription %s could not be processed. Message: %s', $subscription->id, $result->message));
					return ['result'=>'failure', 'redirect'=>''];
				}
			}
				catch(Exception $e){
					wc_add_notice(__('Your payment could not be processed. Please verify your entries and try again.',
						'Braintree_Woocommerce'), 'error');
					$log->writeToLogFile($e);
					return ['result'=>'failure', 'redirect'=>''];
				}
			}
		
	
		$order->update_status(self::$order_status);
		update_post_meta($order->id, '_subscription_type', 'woocommerce');
		
	if($result->transaction){
		self::save_transaction_meta_to_subscription($result, $order);
	}
	elseif($result->paymentMethod){
		self::save_recurring_paymentMethod_meta($result->paymentMethod, $order);
	}
		
		return ['result'=>'success', 'redirect'=>$gateway->get_return_url($order)];
	}
	
	private static function initialize_braintree_recurringPayment(WC_Order $order, $gateway){
		$log = new BraintreeTransactionLog();
		
		$use_token = (isset($_POST['use_saved_card'])) ? true : false;
		
		$nonce = (isset($_POST['payment_method_nonce'])) ? $_POST['payment_method_nonce'] : false;
		
		//Set the subscription id equal to the order id for easy reference.
		$args = array('merchantAccountId'=>self::$merchantAccountId
		);
		$user_id = wp_get_current_user()->ID;
		$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
		
		if($user_id && !$use_token){
				
			if(empty($braintree_id) || !$braintree_id && $nonce){
				try{
					$result = Braintree_Customer::create(array(
							'paymentMethodNonce'=>$nonce));
		
					if($result->success){
						$braintree_id = $result->customer->id;
						update_user_meta($user_id, 'braintree_vault_id', $braintree_id);
						$args['customerId'] = $braintree_id;
						$args['paymentMethodToken'] = $result->customer->paymentMethods[0]->token;
					}
					elseif(! $result->success){
						wc_add_notice(__('Your subsciption could not be created at this time.', 'Braintree_Woocommerce'), 'error');
						return array('result'=>'failure',
								'redirect'=>''
						);
					}
		
				} catch(Exception $e){
					wc_add_notice(__('Your subsciption could not be created at this time.', 'Braintree_Woocommerce'), 'error');
					$log->writeToLogFile('Braintree customer creation failed during subscription checkout.');
					return array('result'=>'failure',
							'redirect'=>''
					);
				}
		
			}
			else {
				try {
					$result = Braintree_PaymentMethod::create(array(
							'paymentMethodNonce'=>$nonce,
							'customerId'=>$braintree_id,
							'options'=>array('failOnDuplicatePaymentMethod'=>true)
					));
					if($result->success){
						$args['paymentMethodToken'] = $result->paymentMethod->token;
					}
					elseif(! $result->success){
						//check if the nonce represents a duplicate card. If so, the nonce can be used in a subscription.
						foreach($result->errors->deepAll() as $error){
							if($error->code == '81724'){
								$args['paymentMethodNonce'] = $nonce;
							}
						}
					}
				}
				catch (Exception $e){
					wc_add_notice(__('Your payment method could not be charged at this time.
							 Please try again', 'Braintree_Woocommerce'), 'error');
					$log->writeToLogFile(sprintf('Message: %s, CustomerId: %s', $e->getMessage(), $braintree_id));
					return array('result'=>'failure',
							'redirect'=>''
					);
				}
					
			}
		}
		else {
			//fetch the payment token from using the array index;
			$token = ($use_token) ? $_POST['existing_card'] : null;
			try {
				$customer = Braintree_Customer::find($braintree_id);
					
				$token = $customer->paymentMethods[$token]->token;
					
				$args['paymentMethodToken'] = $token;
					
			}
			catch(Braintree_Exception_NotFound $e){
				wc_add_notice(__('Your customer Id could not be verified. Please contact the site owner.', 'Braintree_Woocomerce'), 'error');
				$log->writeToLogFile(sprintf('Customer Id %s could not be found in your Braintree vault', $braintree_id));
				return array('result'=>'failure',
						'redirect'=>'');
			}
				
		}
		$subscriptions = wcs_get_subscriptions_for_order($order->id);
		//Process each subscription in the cart. Subscriptions can have multiple items if the items share the same billing schedule, trial period, etc.
		foreach($subscriptions as $subscription){
			$plan_id = self::get_braintree_subscription_planId($subscription);
			
			$args['id'] = self::$order_prefix.$subscription->id;
					
			$args['planId'] = $plan_id;
			
			$price_per_period = $subscription->get_total();
			$subsc_length = self::get_subscription_length($subscription);
			$subsc_unit = $subscription->billing_period;
			
			
			//numberofBillingCycles must be in months because Braintree only supports months for their billing cycles.
			switch($subsc_unit){
				case 'week':
					$subsc_unit = 'month';
					$subsc_length = round(($subsc_length / 52) * 12, 0, PHP_ROUND_HALF_UP); //convert weeks to months and round to the nearest month.
					$price_per_period = round($price_per_period * 52 / 12, 2, PHP_ROUND_HALF_UP); //$5/week * 52weeks / 12 months = price/month
					break;
			
				case 'day':
					$subsc_unit = 'month';
					$subsc_length = round(($subsc_length / 365) * 12, 0, PHP_ROUND_HALF_UP); //convert days to months and round to the nearest month.
					$price_per_period = round($price_per_period * 7 * 52 / 12, 2, PHP_ROUND_HALF_UP); // convert days to weeks to months
					break;
			
				case 'year':
					$subsc_unit = 'month';
					$subsc_length = ($subsc_length * 12 ); //convert year(s) to months.
					$price_per_period = round($price_per_period / 12, 2, PHP_ROUND_HALF_UP);
					break;
			}
			if($subsc_length != 0 || $subsc_length != null){
				$args['numberOfBillingCycles'] = $subsc_length;
			}
			else {
				$args['neverExpires'] = true;
			}
			$args['price'] = $price_per_period;
			
			
			//$signup_fee = WC_Subscriptions_Order::get_sign_up_fee($order);
			
			if(! $start_date = $subscription->get_date('trial_end')){
				$start_date = $subscription->get_date('start');
			}
			$args['firstBillingDate'] = $start_date;
			
			try {
				$result = Braintree_Subscription::create($args);
				
				if($result->success){
					
					self::remove_subscription_from_cart($subscription);
					
					if(self::$order_status == 'wc-completed'){
						$subscription->payment_complete();
						update_post_meta($subscription->id, '_transaction_id', $result->subscription->transactions[0]->id);
					}
					else {
						$subscription->update_status(self::$order_status, __('Credit card payment processed', 'woocommerce'));
						update_post_meta($subscription-id, '_transaction_id', $result->subscription->transactions[0]->id);
					}
				
					self::save_subscription_meta($result, $subscription);
					update_post_meta($subscription->id, '_subscription_type', 'braintree');
					
					if($user_id && !$braintree_id){
						$new_id = $result->subscription->transactions[0]->customerDetails->id;
						update_user_meta($user_id, 'braintree_vault_id', $new_id);
					}
					
					//This is a loop so other subscriptions may be created. Since nonces can only be used once, unset the nonce and add the paymentMethodToken
					unset($args['paymentMethodNonce']);
					
					$args['paymentMethodToken'] = $result->subscription->paymentMethodToken;
					
					wc_add_notice(__(sprintf('Subscription %s was successfully processed.', $subscription->id)), 'Braintree_Woocommerce');
					if(! empty($result->subscription->transactions)){
						$log->writeTransactionToLog($result->subscription->transactions[0], 'Subscription');
					}
					else {
						$log->writeToLogFile(sprintf('Subscription %s was processed successfully. Order ID: %s', $result->subscription->id, $order->id));
					}
				}
				if(! $result->success){
					$log->writeToLogFile($result->message);
			
					if($result->errors){
							
						foreach($result->errors->deepAll() as $error)
							wc_add_notice($error->message, 'error');
					}
					else {
						wc_add_notice(__('Your payment method could not be verified.', 'Braintree_Woocommerce'), 'error');
					}
					return ['result'=>'failure', 'redirect'=>''];
				}
			}
			catch (Exception $e){
				$log->writeToLogFile($e);
				return ['result'=>'failure', 'redirect'=>''];
			}
		}
		
		self::save_subscription_meta($result, $order);
		update_post_meta($order->id, '_subscription_type', 'braintree');
		return array(
				'result'=>'success',
				'redirect'=>$gateway->get_return_url($order));
			
	}
	
	private static function get_braintree_subscription_planId(WC_Subscription $subscription){
		$items = $subscription->get_items();
		//Return the first planId.
		if(count($items) >= 1){
			foreach($items as $item) {
				return get_post_meta($item['product_id'], 'braintree_subscription_id', true);
			}
		}
	}
	
	/**
	 * Takes the order and product id as parameters. Uses the order id as the subscription id and calls the Braintree API
	 * to cancel the subscription.
	 * @param WC_Order $order
	 * @param unknown $product_id
	 */
	public static function cancel_braintree_subscription(WC_Subscription $subscription){
		//Braintree config has been initialized in the init() method.
		//Check if this is a payment method update. If so, return without executing code. 
		$log = new BraintreeTransactionLog();
		if('woocommerce' === get_post_meta($subscription->id, '_subscription_type', true)){
			return false;
		}
		$subscription_id = get_post_meta($subscription->id, '_subscription_id', true);
		
		if(WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment){
			return false;
		}
		
		if (empty($subscription_id) && 'braintree' === get_post_meta($subscription->id, '_subscription_type', true)) {
			$log->writeToLogFile(sprintf('Could not find subscription id attribute of Order %s', $subscription->id));
			return false;
		}
		
		try{
			$result = Braintree_Subscription::cancel($subscription_id);
			if($result->success){
				$log->writeToLogFile(sprintf('Subscription %s has been cancelled.', $subscription->id));
				$subscription->add_order_note(sprintf('Subscription %s has been cancelled.', $subscription->id));
				$subscription->update_status('cancelled');
				return;
			}
			else {
				$log->writeToLogFile(sprintf('There was an error cancelling subscription %s. Message: %s', $subscription->id, $result->message));		
			}
		}
		catch(Braintree_Exception_NotFound $e){
			
			$log->writeToLogFile($e);
		}
		catch(Exception $e){
			$log->writeToLogFile(sprintf('An exception was thrown while cancelling subscription %s. Message: %s', $subscription->id, $e->getMessage()));
		}
		
	}
	
	public static function update_subscription_recurring_amount($order_id, $items){
		if(! isset($items['_order_total'])){
			return false;
		}
		
		if(! $subscription_id = get_post_meta($order_id, '_subscription_id', true)){
			return false;
		}
		
		if(! self::$init_called){
			self::$init();
		}
		
		try{
			$subscription = Braintree_Subscription::find($subscription_id);
				if($subscription instanceof Braintree_Subscription){
					$subsc_amount = $subscription->price;
				}
				else {
					self::$log->writeToLogFile(sprintf('Subscription %s could not be found in your Braintree vault.', $subscription_id));
					return false;
				}
			
		} catch(Exception $e){
			self::$log->writeToLogFile(sprintf('Subscription %s could not be found in your Braintree vault.', $subscription_id));
			return false;
		}
		
		if($items['_order_total'] == $subsc_amount){
			return false; //the amounts match between wordpress and braintree therefore no change was made to the recurring amount. 
		}
		$attrs = array('price' => $items['_order_total']);
		try{
			$result = Braintree_Subscription::update($subscription_id, $attrs);
				if($result->success){
					self::$log->writeToLogFile(sprintf('Braintree subscription recurring amount was updated: Subscription ID: %s, Amount: %s', $subscription_id, $items['_order_total']));
					return;
				}
				else {
					self::$log->writeToLogFile(sprintf('There was an error updating subscription %s. Message: %s', $subscription_id, $result->message));
					return;
				}
		} catch(Exception $e){
			self::$log->add('braintree', 'There was an error while updating the recurring amount
					for subscription '.$order->id.' in your braintree vault.');
			return;
		}
	}
	
	public static function change_braintree_subscription_paymentMethod(WC_Subscription $subscription, $old_payment_method){
		//change the payment method for the subscription
		$log = new BraintreeTransactionLog();
		$attrs = array();
		$use_token = (isset($_POST['use_saved_card'])) ? true : false;
		
		$nonce = (isset($_POST['payment_method_nonce'])) ? $_POST['payment_method_nonce'] : false;
		
		//Set the merchantAccountId for this subscription.
		$attrs['merchantAccountId'] = self::$merchantAccountId;
		
		$user_id = wp_get_current_user()->ID;
		$braintree_id = get_user_meta($user_id, 'braintree_vault_id', true);
		
		$braintree_subscription_id = get_post_meta($subscription->id, '_subscription_id', true);
		
		//If customer isn't using a saved payment method, need to add the new payment method to the vault
		if(!$use_token){
		try {
					$result = Braintree_PaymentMethod::create(array(
							'paymentMethodNonce'=>$nonce,
							'customerId'=>$braintree_id,
							'options'=>array('failOnDuplicatePaymentMethod'=>true),
							'billingAddress'=>array('streetAddress'=>$_POST['billing_address1'],
													'countryCodeAlpha2'=>$_POST['billing_country'],
													'postalCode'=>$_POST['billing_postalcode'])
							
					));
					if($result->success){
						$attrs['paymentMethodToken'] = $result->paymentMethod->token;
						$token = $result->paymentMethod->token;
						$log->writeToLogFile(sprintf('New payment method added for User %s. Payment Method Token: %s', $user_id, $result->paymentMethod->token));
					}
					elseif(! $result->success){
						//check if the nonce represents a duplicate card. If so, the nonce can be used in a subscription.
						foreach($result->errors->deepAll() as $error){
							if($error->code == '81724'){
								WC_Subscriptions::add_notice(__('You have tried to enter a payment method which already exists.', 'Braintree_Woocommerce'), 'error');
							}
							else {
								$log->writeToLogFile(sprintf('User %s could not change the payment method for subscription %s. Message: %s', $user_id, $subscription->id, $result->message));
								WC_Subscriptions::add_notice($error->message);
							}
						}
						return;
					}
				}
				catch(Exeption $e){
					$log->writeToLogFile(sprintf('There was an exception thrown while User %s was changing the payment method for subscription %s', $user_id, $subscription->id));
					wc_add_notice(__('Your payment method could not be charged at this time.
							 Please try again', 'Braintree_Woocommerce'), 'error');
						return;
				}
		}
		//get the payment method token and use it for the new subscription payment method.
		else {
			$token_index = $_POST['existing_card'];
			$customer = Braintree_Customer::find($braintree_id);
			$token = $customer->paymentMethods[$token_index]->token;
			$attrs['paymentMethodToken'] = $token;
		}
		if( 'woocommerce' === get_post_meta($subscription->id, '_subscription_type', true)) {
			$paymentMethod = Braintree_PaymentMethod::find($token);
			self::save_recurring_paymentMethod_meta($paymentMethod, $subscription);
			/* if($subscription->order){
				self::save_recurring_paymentMethod_meta($paymentMethod, $subscription->order);
			} */
		}
		else {
		try {
			$result = Braintree_Subscription::update($braintree_subscription_id, $attrs);
				if($result->success){
					WC_Subscriptions::add_notice(__('Your payment method for subscription '.$subscription->id.' was updated successfully', 'Braintree_Woocommerce'), 'success');
					
					self::save_subscription_meta($result, $subscription);
					
					$log->writeToLogFile(sprintf('Payment method updated for subscription %s. User Id: %s. Payment Method: %s', $subscription->id, $user_id, $payment_details));
					return;
				}
				elseif($result->errors || ! $result->success){
					foreach($result->errors->deepAll() as $error){
						WC_Subscriptions::add_notice($error->message, 'error');
					}
					$log->writeToLogFile($result->message);
				}
		}
		catch(Exception $e){
			$log->writeToLogFile(sprintf('An exception was thrown while updating the payment method for subscription %s. Message: %s', $subscription->id, $e->getMessage()));
			WC_Subscriptions::add_notice(__('Your payment method for subscription '.$subscription->id.' could not be changed at this time.', 'Braintree_Woocommerce'), 'error');
			return;
		}
		}
	}
	
	public static function get_paymentMethod_change_redirectUrl(){
		return get_permalink( self::$config['subscription_config']['change_paymentmethod_redirect_page'] );
	}
	
	/**
	 * Process the recurring payment on the subscription using the saved payment method token. For whatever reason, woocommerce subscriptions
	 * decided to only pass the WC_Order object which causes a disconnect between the Order and the Subscription. This disconnect happends when a payment method
	 * on a subscription is updated. However, subscriptions expects to use the meta data of the WC_Object. 
	 * @param $amount_to_charge
	 * @param WC_Subscription $subscription
	 * @param string $product_id
	 */
	public static function process_recurring_subscriptionPayment($amount, WC_Order $order){
		if('braintree' === get_post_meta($order->id, '_subscription_type', true)){
			return;
		}
		$log = new BraintreeTransactionLog();
		
		//get the subscription object so the payment method for the subscription will be charged.
		$post = get_post($_GET['post_id']);
		$array = json_decode($post->post_content, true);
		$subscriptionId = $array['subscription_id'];
		$subscription = wcs_get_subscription($subscriptionId);
		
		if(!$payment_token = get_post_meta($subscription->id, '_payment_method_token', true)){
			$log->writeToLogFile('Subscription: '.$subscription->id.' Status: Failed, Reason: No Payment Method Token');
			return;
		}
		if($amount == 0){
			$subscription->payment_complete();
			return true;
		}
		try {
			$attribs = ['merchantAccountId'=>self::$merchantAccountId, 
						'amount'=>$amount,
						'paymentMethodToken'=>$payment_token, 
						'options'=>['submitForSettlement'=>true], 
						'billing'=>['firstName'=>$subscription->billing_first_name, 
									'lastName'=>$subscription->billing_last_name, 
									'postalCode'=>$subscription->billing_postcode, 
									'streetAddress'=>$subscription->billing_address_1
						] 
			];
			$result = Braintree_Transaction::sale($attribs);
			
			if(! $result->success){
				$log->writeFailedTransactionToLog($result, 'Subscription Payment');
				$subscription->update_status('failed', sprintf('Payment for subscription %s failed. Error message: %s', $subscription->id, $result->message));
				$subscription->add_order_note(sprintf('Payment for subscription %s failed. Error message: %s', $subscription->id, $result->message));
				return false;
			}
			else if ($result->success){
				$log->writeTransactionToLog($result->transaction, 'Subscription Payment');
				update_post_meta($subscription->id, '_transaction_id', $result->transaction->id);
				$order->add_order_note(sprintf('Subscription payment processed. Subscription Id: %s, Transaction Id: %s', $subscription->id, $result->transaction->id));
				return true;
			}
		}
		catch(Braintree_Exception_Authentication $e){
			$log->writeToLogFile(sprintf('Payment for subscription %s failed due to an Authentication. Check your API keys', $order->id));
			$order->update_status('failed', sprintf('Subscription payment failed. %s', $e->getMessage()));
			return false;
		}
		catch(Braintree_Exception_Authorization $e){
			$log->writeToLogFile(sprintf('Payment for subscription %s failed due to an Authorization error. Check your API keys.', $order->id));
			$order->update_status('failed', sprintf('Subscription payment failed. %s', $e->getMessage()));
			return false;
		}
		catch(Exception $e){
			$log->writeToLogFile($e);
			$order->update_status('failed', sprintf('Subscription payment failed. Error message is: %s', $e->getMessage()));
			return false;
		}
	}
	/**
	 * Returns the payment method that was used to process the subscription.
	 * @param string $payment_method_to_display
	 * @param WC_Subscription $subscription
	 * @return string $payment_method_to_display
	 */
	public static function my_subscriptions_payment_method($payment_method_to_display, WC_Subscription $subscription){
		
		if($payment_method_title = get_post_meta($subscription->id, '_recurring_payment_method_title', true)){
			$payment_method_to_display = $payment_method_title;
		}
		
		return $payment_method_to_display;
	}
	
	public static function failing_payment_method_updated(WC_Subscription $subscription){
		
	}

	public static function save_subscription_meta($result, WC_Order $subscription){
		update_post_meta($subscription->id, '_payment_method_token', $result->subscription->paymentMethodToken);
		
		update_post_meta($subscription->id, '_subscription_id', $result->subscription->id);
			
		if($result->subscription->transactions[0]->paymentInstrumentType == 'credit_card'){
			$cardDetails = $result->subscription->transactions[0]->creditCardDetails;
			update_post_meta($subscription->id, '_payment_method_title', $cardDetails->cardType.' - '.$cardDetails->maskedNumber);
			update_post_meta($subscription->id, '_recurring_payment_method_title', $cardDetails->cardType.' - '.$cardDetails->maskedNumber);
		}
		elseif($result->subscription->transactions[0]->paymentInstrumentType == 'paypal_account') {
			$paypalDetails = $result->subscription->transactions[0]->paypalDetails;
			update_post_meta($subscription->id, '_payment_method_title', 'PayPal - '.$paypalDetails->payerEmail);
			update_post_meta($subscription->id, '_recurring_payment_method_title', 'PayPal - '.$paypalDetails->payerEmail);
		}
	}
	
	public static function save_recurring_paymentMethod_meta($paymentMethod, WC_Order $subscription){
		update_post_meta($subscription->id, '_payment_method_token', $paymentMethod->token);
		
		if(is_a($paymentMethod, Braintree_CreditCard)){
			update_post_meta($subscription->id, '_payment_method_title', $paymentMethod->cardType.' - '.$paymentMethod->maskedNumber);
			update_post_meta($subscription->id, '_recurring_payment_method_title', $paymentMethod->cardType.' - '.$paymentMethod->maskedNumber);
			self::$log->writeToLogFile(sprintf('Payment method saved for subscription. Subscription ID: %s, Payment Method: %s', $subscription->id, $paymentMethod->cardType.' - '.$paymentMethod->maskedNumber));
		}
		elseif (is_a($paymentMethod, Braintree_PayPalAccount)){
			update_post_meta($subscription->id, '_payment_method_title', 'PayPal - '.$paymentMethod->email);
			update_post_meta($subscription->id, '_recurring_payment_method_title', 'PayPal - '.$paymentMethod->email);
			self::$log->writeToLogFile(sprintf('Payment method saved for subscription. Subscription ID: %s, Payment Method: %s', $subscription->id, 'PayPal - '.$paymentMethod->email));
		}
	}
	
	public static function save_transaction_meta_to_subscription($result, WC_Order $subscription){
		
		update_post_meta($subscription->id, '_transaction_id', $result->transaction->id);
		
		
		if($result->transaction->paymentInstrumentType == 'credit_card'){
			$cardDetails = $result->transaction->creditCardDetails;
			update_post_meta($subscription->id, '_payment_method_token', $cardDetails->token);
			update_post_meta($subscription->id, '_payment_method_title', $cardDetails->cardType.' - '.$cardDetails->maskedNumber);
			update_post_meta($subscription->id, '_recurring_payment_method_title', $cardDetails->cardType.' - '.$cardDetails->maskedNumber);
		}
		elseif ($result->transaction->paymentInstrumentType == 'paypal_account'){
			$paypalDetails = $result->transaction->paypalDetails;
			update_post_meta($subscription->id, '_payment_method_token', $paypalDetails->token);
			update_post_meta($subscription->id, '_payment_method_title', 'PayPal - '.$paypalDetails->payerEmail);
			update_post_meta($subscription->id, '_recurring_payment_method_title', 'PayPal - '.$paypalDetails->payerEmail);
		}
	}
	
	public static function get_subscription_length(WC_Subscription $subscription){
		$items = $subscription->get_items();
		foreach($items as $item){
			$product_id = $subscription->get_product_from_item($item);
			break;
		}
	 return get_post_meta($product_id, '_subscription_length', true);
	}
	
	public static function remove_subscription_from_cart(WC_Subscription $subscription){

		$cart = WC()->cart->get_cart();
			
		foreach( $cart as $cart_key => $item ){
			$product_id = $item['product_id'];
			foreach($subscription->get_items() as $item) {
				if($item['product_id'] == $product_id) {
					WC()->cart->remove_cart_item($cart_key);
				}
			}
		}
	}
	
	public function create_subscription_renewal_order(WC_Order $order, WC_Subscription $subscription){
		
	}
	
	public static function admin_change_subscription_paymentMethod($payment_meta, WC_Subscription $subscription){
		$id = self::$id;
		$payment_meta[$id] = array('post_meta'=>array(
				'_payment_method_token'=>array(
						'label'=>'Payment Method Token', 
						'value'=>get_post_meta($subscription->id, '_payment_method_token', true)
				),
				
		));
		return $payment_meta;
		
	}
	
	/**
	 * Validate the payment meta entered by the Admin on the order edit page.
	 * @param string $payment_method_id
	 * @param array $payment_meta
	 */
	public static function validate_subscription_payment_meta( $payment_method_id, $payment_meta){
		if(self::$id === $payment_method_id){
			if(empty($payment_meta['post_meta']['_payment_method_token']['value'])){
				throw  new Exception('You must provide a payment method token.');
			}
			$post_id = $_POST['post_ID'];
			$token = $payment_meta['post_meta']['_payment_method_token']['value'];
	
			$result = Braintree_PaymentMethod::find($token);
			
			if(is_a($result, Braintree_Base)) {
				$subscriptionType = get_post_meta($post_id, '_subscription_type', true);
				if('braintree' === $subscriptionType){
					$log = new BraintreeTransactionLog();
					
						$subscriptionId = get_post_meta($post_id, '_subscription_id', true);
						$result = Braintree_Subscription::update($subscriptionId,
								array('paymentMethodToken'=>$meta_value));
						if($result->success){
							$log->writeTransactionToLog($result->subscription->transactions[0], 'Subscription Update');
						}
						elseif(! $result->success) {
							$log->writeToLogFile($result->message);
							throw new Exception($result->message);
						}					
				}
			}
			else {
				throw new Exception('The payment method token is not valid');
			}
			
			return $payment_meta;
			}
			
	}
	
	/**
	 * Updates the payment meta for the woocommerce subscription.
	 * @param WC_Order $subscription
	 * @param string $table
	 * @param string $meta_key
	 * @param string $meta_value
	 */
	public static function save_admin_payment_meta(WC_Order $subscription, $table, $meta_key, $meta_value){
		$log = new BraintreeTransactionLog();
		$subscription_type = get_post_meta($subscription->id, '_subscription_type', true);
		$subscriptionId = get_post_meta($subscription->id, '_subscription_id', true);
		if($meta_key === '_payment_method_token'){
			
			try {
				$paymentMethod = Braintree_PaymentMethod::find($meta_value);
			}
			catch(Braintree_Exception_NotFound $e){
				$log->writeToLogFile($e);
				return false;
			}
			catch(Exception $e){
				$log->writeToLogFile($e);
				return false;
			}
			
			/* if($subscription->order){
				self::save_recurring_paymentMethod_meta($paymentMethod, $subscription->order);
			} */
			self::save_recurring_paymentMethod_meta($paymentMethod, $subscription);
			
		}
		return true;
	}
		
}
?>