<?php

//Update selected options using ajax
class WooCommerce_Product_Options_Ajax {

    /**
     * Creates object
     */
    function __construct() {
        //ajax
        add_action( 'wp_ajax_nopriv_update_product_option', array( $this, 'ajax_update_product_option' ) );
        add_action( 'wp_ajax_update_product_option', array( $this, 'ajax_update_product_option' ) );
    }

    function ajax_update_product_option() {
        print $this->update_product_option();
        die();
    }

    /**
     * ajax - gets the price
     */
    function update_product_option( $post_id = -1 /* The product id */, $product_option_id = -1 /* The option id of the product option (1, 2, 3 depending on whether it is the 1st/2nd/3rd option) */, $option = '' /* The value of the option */, $product_option_post_id = -1 /* Either the product id or the product group post id */, $variation_id = -1 /* The selected variation id */ ) {
        if ( $product_option_post_id == -1 ) {
            $product_option_post_id = $post_id;
        }
        if ( !empty( $_POST ) && !empty( $_POST['product_option_id'] ) ) {
            $post_id = intval( $_POST['post_id'] );
            $product_option_post_id = intval( $_POST['product_option_post_id'] );
            $product_option_id = intval( $_POST['product_option_id'] );
            $option = value( $_POST, 'product_option', '' );
            $variation_id = value( $_POST, 'variation_id', '' );
        }
        if ( $product_option_id == -1 ) {
            return;
        }
        if ( is_string( $option ) ) {
            $product_option = wp_kses_data( $option );
        } elseif ( is_array( $option ) ) {
            $product_option = array_map( 'wp_kses_data', $option );
        }
        $src = '';
        $product_options = get_post_meta( $product_option_post_id, 'backend-product-options', true );
        if ( $product_options[$product_option_id]['type'] == 'image_upload' ) {
            $src = $this->upload_image( 'product-options[' . $product_option_id . ']' );
        } else {
            $_SESSION['product_options_' . $product_option_post_id . '_' . $product_option_id] = $product_option;
        }
        ob_start();
        $price = $this->get_price( $post_id );
        if ( !empty( $variation_id ) && $variation_id != -1 ) {
            $price = $this->get_price( $variation_id );
        }
        $_SESSION['product_options_' . $post_id . '_price'] = $price;
        $product = get_product( $post_id );
        $price_html = '<span class="amount">' . $this->get_price_html( $price, $product ) . '</span>';
        if ( $product && $product->is_on_sale() ) {
            $sale_price_difference = floatval( $product->get_regular_price() ) - floatval( $product->get_sale_price() );
            $price_html = $product->get_price_html_from_to( $price + $sale_price_difference, $price );
        }
        $response = array(
            'price' => $price_html,
            'option_price' => woocommerce_price( $this->get_option_price( $post_id, $product_option_post_id, $product_option_id, $product_options[$product_option_id] ) ) . ob_get_clean(),
            'src' => $src,
        );
        return json_encode( $response );
    }

    function get_price_html( $price, $product ) {
        $price = wc_format_localized_price( wc_price( $price ) ) . $product->get_price_suffix();
        $price = apply_filters( 'woocommerce_price_html', $price, $product );
        return apply_filters( 'woocommerce_get_price_html', $price, $product );
    }

    function upload_image( $name ) {
        require_once(ABSPATH . "wp-admin" . '/includes/image.php');
        require_once(ABSPATH . "wp-admin" . '/includes/file.php');
        require_once(ABSPATH . "wp-admin" . '/includes/media.php');
        $post_id = intval( $_POST['post-id'] );
        $option_id = intval( $_POST['option-id'] );
        $options = get_post_meta( $post_id, 'backend-product-options', true );
        $option = $options[$option_id];
        if ( !empty( $option['multiple'] ) ) {
            
        } else {
            $file = strval( $_POST['file_name'] );
            $post_id = 0;
            $attachment_id = media_handle_upload( $file, $post_id );
            $src = wp_get_attachment_url( $attachment_id );
            $_SESSION['product_options_' . $post_id . '_' . $product_option_id] = $src;
            return wp_get_attachment_url( $attachment_id );
        }
    }

    /**
     * Converts number to a float
     */
    function convert_to_number( $number ) {
        return floatval( $number );
    }

    /**
     * Gets the price of one option
     */
    function get_option_price( $product_post_id, $post_id, $product_option_id, $product_option ) {
        $product = get_product( $product_post_id );
        if ( $product ) {
            $product_price = floatval( $product->get_price() );
        } else {
            $product_price = 0;
        }
        $value = value( $product_option, 'default', '' );
        $type = $product_option['type'];
        $multiple = value( $product_option, 'multiple', '' );
        if ( $type == 'multi_select' ) {
            $value = array( $value );
        }
        if ( isset( $_SESSION['product_options_' . $post_id . '_' . $product_option_id] ) ) {
            $value = $_SESSION['product_options_' . $post_id . '_' . $product_option_id];
        }
        $option_price = 0;
        if ( $type == 'radio' || $type == 'select' || ($type == 'radio_image' && empty( $multiple )) ) {
            if ( !empty( $product_option['options'] ) && !empty( $product_option['options'][$value] ) ) {
                if ( !empty( $product_option['options'][$value]['price'] ) ) {
                    $option_price = floatval( $product_option['options'][$value]['price'] );
                }
                if ( !empty( $product_option['options'][$value]['percentage_price'] ) ) {
                    $option_price = $option_price + $product_price * floatval( $product_option['options'][$value]['percentage_price'] ) / 100;
                }
            }
        } elseif ( $type == 'multi_select' || $type == 'radio_image' || $type == 'checkboxes' ) {
            if ( !empty( $value ) ) {
                if ( is_string( $value ) ) {
                    $value = array( $value );
                }
                foreach ( $value as $selected_option ) {
                    if ( !empty( $product_option['options'] ) && !empty( $product_option['options'][$selected_option] ) && !empty( $product_option['options'][$selected_option]['price'] ) ) {
                        $option_price = $option_price + floatval( $product_option['options'][$selected_option]['price'] );
                    }
                }
            }
        } elseif ( $type == 'text' || $type == 'textarea' ) {
            $characters = strlen( $value );
            if ( !empty( $value ) && !empty( $product_option['price_if_not_empty'] ) ) {
                $option_price = floatval( $product_option['price_if_not_empty'] );
            }
            if ( !empty( $product_option['price_per_character'] ) ) {
                $price_per_character = floatval( $product_option['price_per_character'] );
                $option_price = $option_price + $price_per_character * $characters;
            }
            $number_of_lower_case_letters = 0;
            $number_of_upper_case_letters = 0;
            for ( $i = 0; $i < strlen( $value ); $i++ ) {
                if ( ctype_lower( $value[$i] ) ) {
                    $number_of_lower_case_letters = $number_of_lower_case_letters + 1;
                } elseif ( ctype_upper( $value[$i] ) ) {
                    $number_of_upper_case_letters = $number_of_upper_case_letters + 1;
                }
            }
            if ( !empty( $product_option['price_per_lower_case_letter'] ) ) {
                $price_per_lower_case_letter = floatval( $product_option['price_per_lower_case_letter'] );
                $option_price = $option_price + $price_per_lower_case_letter * $number_of_lower_case_letters;
            }
            if ( !empty( $product_option['price_per_upper_case_letter'] ) ) {
                $price_per_upper_case_letter = floatval( $product_option['price_per_upper_case_letter'] );
                $option_price = $option_price + $price_per_upper_case_letter * $number_of_upper_case_letters;
            }
        } elseif ( $type == 'number' || $type == 'number_as_text' || $type == 'range' ) {
            $starting_from = 0;
            $percentage_price = 100;
            if ( !empty( $product_option['starting_from'] ) ) {
                $starting_from = $product_option['starting_from'];
            }
            if ( isset( $product_option['percentage_price'] ) ) {
                $percentage_price = $product_option['percentage_price'];
            }
            $option_price = (floatval( $value ) - floatval( $starting_from )) * floatval( $percentage_price ) / 100;
        } elseif ( $type == 'checkbox' ) {
            if ( empty( $value ) ) {
                $option_price = floatval( value( $product_option, 'unchecked_price', 0 ) );
            } else {
                $option_price = floatval( value( $product_option, 'checked_price', 0 ) );
            }
        }
        return $option_price;
    }

    /**
     * Gets the price of the product with id $post_id including all the options
     */
    function get_price( $post_id ) {
        $product_options = get_post_meta( $post_id, 'backend-product-options', true );
        $product = get_product( $post_id );
        $price = 0;
        if ( $product ) {
            $price = floatval( $product->get_price() );
        }
        if ( !empty( $_POST['fpd_product_price'] ) ) {
            $price = floatval( $_POST['fpd_product_price'] );
        }
        if ( $product->product_type == 'variable' ) {
            //Get the attributes
            $product_attributes = array();
            if ( !empty( $_POST['attributes'] ) ) {
                foreach ( $_POST['attributes'] as $attribute_name => $attribute_value ) {
                    $product_attributes[$attribute_name] = $attribute_value;
                }
                $_SESSION['attributes'][$post_id] = $product_attributes;
            } elseif ( !empty( $_SESSION['attributes'][$post_id] ) ) {
                $product_attributes = $_SESSION['attributes'][$post_id];
            }
            //Get the variation from the attributes
            $child_products = $product->get_children();
            if ( !empty( $child_products ) ) {
                foreach ( $child_products as $child_id ) {
                    $child_product = get_product( $child_id );
                    if ( $child_product && $child_product->is_visible() ) {
                        $child_attributes = get_post_meta( $child_id );
                        $attribute_matches = true;
                        if ( !empty( $product_attributes ) ) {
                            foreach ( $product_attributes as $product_attribute_name => $product_attribute_value ) {
                                if ( empty( $child_attributes[$product_attribute_name] ) || empty( $child_attributes[$product_attribute_name][0] ) || $child_attributes[$product_attribute_name][0] != $product_attribute_value ) {
                                    $attribute_matches = false;
                                    break;
                                }
                            }
                        }
                        if ( $attribute_matches ) {
                            $price = floatval( $child_product->get_price() );
                            break;
                        }
                    }
                }
            }
        }
        if ( !empty( $product_options ) ) {
            foreach ( $product_options as $product_option_id => $product_option ) {
                $option_price = $this->get_option_price( $post_id, $post_id, $product_option_id, $product_option );
                $price = $price + floatval( $option_price );
            }
        }
        global $woocommerce_product_options_product_option_group;
        $price = $price + $woocommerce_product_options_product_option_group->group_option_price( $post_id );
        return $price;
    }

}

$woocommerce_product_options_ajax = new WooCommerce_Product_Options_Ajax();
?>