<?php

/**
 * Plugin Name: WooCommerce Product Options (shared on wplocker.com)
 * Plugin URI: http://codecanyon.net/user/wpshowcase?ref=WPShowCase
 * Description: Add custom options with prices to your WooCommerce products.
 * Author: WPShowCase
 * Version: 2.41
 * Author URI: http://www.wpshowcase.net
 */
if ( !isset( $_SESSION ) ) {
    @session_start();
}

require_once 'includes/functions.php';
require_once 'includes/woocommerce-not-installed-notice.php';
require_once 'includes/product-admin.php';
require_once 'includes/product-group.php';
require_once 'includes/product-frontend.php';
require_once 'includes/product-options-in-cart.php';
require_once 'includes/checkout.php';
require_once 'includes/woocommerce-product-options-ajax.php';
require_once 'includes/settings.php';

class WooCommerce_Product_Options {

    //Creates the object
    function __construct() {
        //Enqueue scripts for frontend
        add_action( 'wp_enqueue_scripts', array( $this, 'frontend_scripts' ) );
        //Ajax for backend
        add_action( 'wp_ajax_get_product_option_preview', array( $this, 'ajax_get_product_option_preview' ) );
        //Multilanguage
        load_plugin_textdomain( 'woocommerce-product-options', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
		// fix scripts/styles for https
		add_action('wp_print_scripts', array( $this, 'change_http_scripts_to_https' ), 200);
		add_action('wp_print_styles', array( $this, 'change_http_styles_to_https' ), 200);
    }
	 
	function change_http_scripts_to_https() {
		if (!empty($_SERVER['HTTPS'])) {
			global $wp_scripts;
			foreach ( (array) $wp_scripts->registered as $script ) {
				if ( stripos($script->src, 'http://', 0) !== false ) {
					$script->src = str_replace('http://', 'https://', $script->src);
				}
			}
		}
	}
 
	function change_http_styles_to_https() {
		if (!empty($_SERVER['HTTPS'])) {
			global $wp_styles;
			foreach ( (array) $wp_styles->registered as $script ) {
				if ( stripos($script->src, 'http://', 0) !== false )
					$script->src = str_replace('http://', 'https://', $script->src);
			}
		}
	}
	
    //Ajax for the backend to preview the frontend
    function ajax_get_product_option_preview() {
        global $woocommerce_product_options_product_frontend;
        $woocommerce_product_options_product_frontend->ajax_get_product_option_preview();
    }

    //Loads frontend css and js
    function frontend_scripts() {
        wp_enqueue_script( 'jquery' );
        wp_enqueue_script( 'jquery' );
        wp_enqueue_script( 'jquery-ui-core' );
        wp_enqueue_script( 'jquery-ui-widget' );
        wp_enqueue_script( 'jquery-ui-mouse' );
        wp_enqueue_script( 'jquery-ui-slider' );
        wp_enqueue_script( 'jquery-ui-button' );
        wp_enqueue_script( 'jquery-ui-spinner' );
        wp_enqueue_script( 'jquery-ui-draggable' );
        wp_enqueue_script( 'jquery-ui-datepicker' );
        wp_enqueue_script( 'jquery-ui-tooltip' );
        wp_enqueue_script( 'wc-add-to-cart-variation' );
        wp_enqueue_script( 'spectrum', WP_PLUGIN_URL . '/woocommerce-product-options/spectrum-master/spectrum.js', array( 'jquery' ), false, true );
        wp_enqueue_style( 'spectrum-css', WP_PLUGIN_URL . '/woocommerce-product-options/spectrum-master/spectrum.css' );
        wp_enqueue_script( 'woocommerce-product-options-frontend', WP_PLUGIN_URL . '/woocommerce-product-options/assets/js/product-options-frontend.js', array( 'jquery', 'wc-add-to-cart-variation', 'jquery-ui-tooltip' ), false, true );
        $localize_array = array( 'ajaxurl' => admin_url( 'admin-ajax.php', 'relative' ),
            'numeric_error' => __( 'Please enter a number', 'woocommerce-product-options' ),
            'required_error' => __( 'Required field - please enter a value', 'woocommerce-product-options' ),
            'number_error' => __( 'Number field - please enter a number', 'woocommerce-product-options' ),
            'min_error' => __( 'Please enter a value more than ', 'woocommerce-product-options' ),
            'max_error' => __( 'Please enter a value less than ', 'woocommerce-product-options' ),
            'checkboxes_max_error' => __( 'You have selected too many values', 'woocommerce-product-options' ),
            'checkboxes_min_error' => __( 'Please select more values', 'woocommerce-product-options' ), );
        wp_localize_script( 'woocommerce-product-options-frontend', 'woocommerce_product_options_settings', $localize_array );
        wp_register_style( 'woocommerce-product-options-frontend-css', WP_PLUGIN_URL . '/woocommerce-product-options/assets/css/product-options-frontend.css' );
        wp_enqueue_style( 'woocommerce-product-options-frontend-css' );
    }

}

//Creates object
$woocommerce_product_options = new WooCommerce_Product_Options();
?>